###########################################################################
#
# This software is governed by a license. See
# LICENSE.txt for the terms of this license.
#
###########################################################################

__metaclass__ = type

def copyTable(norm, table):
    new_table = []
    encoding = norm.encoding
    for key, value in table:
        if not isinstance(key, unicode):
            key = key.decode(encoding, 'strict')
        if not isinstance(value, unicode):
            value = value.decode(encoding, 'strict')
        new_table.append((key, value))
    norm.table = new_table

class Normalizer:

    __slots__ = ('encoding', 'table')

    def __init__(self, table, encoding='latin-1'):
        self.encoding = encoding
        copyTable(self, table)

    def normalize(self, words):
        if isinstance(words, basestring):
            return self._normalizeWord(words)
        result = []
        for word in words:
            result.append(self._normalizeWord(word))
        return result

    def _normalizeWord(self, word):
        if not isinstance(word, unicode):
            word = unicode(word, self.encoding, 'strict')
        for src, dst in self.table:
            word = word.replace(src, dst)
        return word

    def getTable(self):
        return self.table
